==> gtracks-1.10.2.tar.gz: gtracks-1.10.2/gtracks/gff3_to_bed12.py <==
#===============================================================================
# gff3_to_bed12.py
#===============================================================================

"""Convert gff3 data to bed12 format"""




# Imports ======================================================================

from argparse import ArgumentParser
from pybedtools import BedTool
import gzip




# Functions ====================================================================

def parse_gff_attributes(attr):
    """Parse an entry from the "attr" column of a GFF3 file and yield it as
    a dict

    Parameters
    ----------
    attr : str
        feature attribute string

    Returns
    ------
    dict
        attr entries as a dict
    """

    return dict(pair.split('=') for pair in attr.split(';'))


def parse_gff(gff: str, type='gene'):
    """Parse a GFF3 file and yield its lines as tuples

    Parameters
    ----------
    gff : str
        path to GFF3 file
    type
        string indcicating feature type to include, or None to include all
        features
    """

    with (gzip.open(gff, 'rt') if gff.endswith('.gz') else open(gff)) as f:
        for l in f:
             if not l.startswith('#'):
                seqid, _, t, start, end, _, strand, _, attr = l.rstrip().split('\t')
                if ((t == type) or (type is None)):
                    yield (seqid, int(start), int(end), strand,
                           parse_gff_attributes(attr))


def generate_bed(gff, type='gene', canon=False):
    for seqid, start, end, strand, attr in parse_gff(gff, type=type):
        gene_id = attr['ID'] if 'ID' in attr.keys() else attr['Parent'].split('_')[0]
        if attr['Parent'].endswith('_T001') or not canon:
            yield seqid, start, end, gene_id, 0, strand


def parse_arguments():
    parser = ArgumentParser(description='convert gff3 to bed12')
    parser.add_argument('gff', metavar='<input.gff3>', help='input gff3 file')
    parser.add_argument('--canon', action='store_true', help='canonical transcripts only')
    return parser.parse_args()


def main():
    args = parse_arguments()
    genes = BedTool(tuple(generate_bed(args.gff, canon=args.canon))).sort()
    exons = BedTool(tuple(generate_bed(args.gff, type='exon', canon=args.canon))).sort()
    cds = BedTool(tuple(generate_bed(args.gff, type='CDS', canon=args.canon))).sort()
    for gene in genes:
        block_size, block_start = zip(
            *((str(exon.stop - exon.start), str(exon.start - gene.start))
              for exon in exons.intersect(BedTool((gene,)), nonamecheck=True)))
        thick = tuple(cds.intersect(BedTool((gene,)), nonamecheck=True))
        if thick:
            thick_start = thick[0].start
            thick_stop = thick[-1].end
        else:
            thick_start = gene.start
            thick_stop = gene.start
        print('\t'.join(str(x) for x in tuple(gene) + (thick_start, thick_stop,
            '0,0,0', len(block_size), ','.join(block_size)+',',
            ','.join(block_start)+',')))


if __name__ == '__main__':
    main()

==> gtracks-1.10.3.tar.gz: gtracks-1.10.3/gtracks/gff3_to_bed12.py <==
#===============================================================================
# gff3_to_bed12.py
#===============================================================================

"""Convert gff3 data to bed12 format"""




# Imports ======================================================================

from argparse import ArgumentParser
from pybedtools import BedTool
import gzip




# Functions ====================================================================

def parse_gff_attributes(attr):
    """Parse an entry from the "attr" column of a GFF3 file and yield it as
    a dict

    Parameters
    ----------
    attr : str
        feature attribute string

    Returns
    ------
    dict
        attr entries as a dict
    """

    return dict(pair.split('=') for pair in attr.split(';'))


def parse_gff(gff: str, type='gene'):
    """Parse a GFF3 file and yield its lines as tuples

    Parameters
    ----------
    gff : str
        path to GFF3 file
    type
        string indcicating feature type to include, or None to include all
        features
    """

    with (gzip.open(gff, 'rt') if gff.endswith('.gz') else open(gff)) as f:
        for l in f:
             if not l.startswith('#'):
                seqid, _, t, start, end, _, strand, _, attr = l.rstrip().split('\t')
                if ((t == type) or (type is None)):
                    yield (seqid, int(start), int(end), strand,
                           parse_gff_attributes(attr))


def generate_bed(gff, type='gene', canon=False):
    for seqid, start, end, strand, attr in parse_gff(gff, type=type):
        gene_id = attr['ID'] if 'ID' in attr.keys() else attr['Parent'].split('_')[0]
        if not attr.get('Parent'):
            yield seqid, start, end, gene_id, 0, strand
        elif attr['Parent'].endswith('_T001') or not canon:
            yield seqid, start, end, gene_id, 0, strand


def parse_arguments():
    parser = ArgumentParser(description='convert gff3 to bed12')
    parser.add_argument('gff', metavar='<input.gff3>', help='input gff3 file')
    parser.add_argument('--canon', action='store_true', help='canonical transcripts only')
    return parser.parse_args()


def main():
    args = parse_arguments()
    genes = BedTool(tuple(generate_bed(args.gff, canon=args.canon))).sort()
    exons = BedTool(tuple(generate_bed(args.gff, type='exon', canon=args.canon))).sort()
    cds = BedTool(tuple(generate_bed(args.gff, type='CDS', canon=args.canon))).sort()
    for gene in genes:
        block_size, block_start = zip(
            *((str(exon.stop - exon.start), str(exon.start - gene.start))
              for exon in exons.intersect(BedTool((gene,)), nonamecheck=True)))
        thick = tuple(cds.intersect(BedTool((gene,)), nonamecheck=True))
        if thick:
            thick_start = thick[0].start
            thick_stop = thick[-1].end
        else:
            thick_start = gene.start
            thick_stop = gene.start
        print('\t'.join(str(x) for x in tuple(gene) + (thick_start, thick_stop,
            '0,0,0', len(block_size), ','.join(block_size)+',',
            ','.join(block_start)+',')))


if __name__ == '__main__':
    main()
